'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Define functional component. Destructure the props.
var Taggy = function Taggy(_ref) {
    var text = _ref.text,
        spans = _ref.spans,
        ents = _ref.ents;


    // Initialize an empty array that will hold the text and entities
    var jsx = [];

    // METHOD 1 - STRING
    if (typeof text === 'string') {
        // Initialize an empty array. The contents of 'elements' will eventually get pushed to the 'jsx' array, and will be converted to jsx markup in the process.
        var elements = [];
        // Keep track of location in the string of text
        var offset = 0;
        // Loop through the spans, using the span data to construct the 'elements' array
        spans.forEach(function (_ref2) {
            var type = _ref2.type,
                start = _ref2.start,
                end = _ref2.end;

            // Create a string of text that does not contain any entities
            var fragment = text.slice(offset, start);
            // Create an entity
            var entity = text.slice(start, end);
            // Push the both of them to the elements array
            elements.push(fragment);
            elements.push({
                token: entity,
                type: type.toLowerCase()
            });
            // Update our position within the string of text
            offset = end;
        });
        // After pushing all of the entities to the 'elements' array, push the remaining text to the 'elements' array. Elements should now consist of strings and objects/entities.
        elements.push(text.slice(offset, text.length));
        // Filter out unnecessary spaces
        elements = elements.filter(function (val) {
            return val !== ' ';
        });
        // Loop through elements array looking for multi-word entities.
        for (var e = 0; e < elements.length; e++) {
            // Check if we've stopped at an entity
            if (elements[e].token) {
                // Examine the consecutive entities, if any.
                for (var i = e + 1; i < elements.length; i++) {
                    // Combine consecutive entities of the same type into one entity. Then, mark the duplicates as 'false'.
                    if (typeof elements[i] !== 'string' && elements[i].type === elements[e].type) {
                        elements[e].token += ' ' + elements[i].token;
                        elements[i] = false;
                    }
                    // Stop the loop when we've run out of consecutive entities
                    if (typeof elements[i] === 'string') {
                        break;
                    }
                }
            }
        }
        // Filter out the consecutive entities that were marked as duplicates
        elements = elements.filter(function (val) {
            return !!val;
        });
        // Loop through our 'elements' array. Push strings directly to the 'jsx' array. Convert entity objects to jsx markup, then push to the 'jsx' array.
        elements.forEach(function (t) {
            if (typeof t === 'string') {
                jsx.push(t);
            } else {
                jsx.push(_react2.default.createElement(
                    'mark',
                    {
                        style: {
                            padding: '0.25em 0.35em',
                            margin: '0px 0.25em',
                            lineHeight: '1',
                            display: 'inline-block',
                            borderRadius: '0.25em',
                            border: '1px solid',
                            background: 'rgba(\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.r + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.g + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.b + ',\n                                0.2\n                            )',
                            borderColor: 'rgb(\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.r + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.g + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.b + '\n                            )'
                        }
                    },
                    t.token,
                    _react2.default.createElement(
                        'span',
                        {
                            style: {
                                boxSizing: 'border-box',
                                fontSize: '0.6em',
                                lineHeight: '1',
                                padding: '0.35em',
                                borderRadius: '0.35em',
                                textTransform: 'uppercase',
                                display: 'inline-block',
                                verticalAlign: 'middle',
                                margin: '0px 0px 0.1rem 0.5rem',
                                background: 'rgb(\n                                    ' + ents.find(function (ent) {
                                    return ent.type === t.type;
                                }).color.r + ',\n                                    ' + ents.find(function (ent) {
                                    return ent.type === t.type;
                                }).color.g + ',\n                                    ' + ents.find(function (ent) {
                                    return ent.type === t.type;
                                }).color.b + '\n                                )'
                            }
                        },
                        t.type
                    )
                ));
            }
        });
    }

    // METHOD 2 - TOKENS
    if (Array.isArray(text)) {
        // Rename 'text' to 'tokens' for clarity
        var tokens = text;
        // Loop through the 'spans' array. Use the span data to update our 'tokens' array with entities
        for (var s = 0; s < spans.length; s++) {
            tokens[spans[s].index] = {
                token: tokens[spans[s].index],
                type: spans[s].type.toLowerCase()
            };
        }
        // Loop through the tokens array, looking for multi-word entities
        for (var t = 0; t < tokens.length; t++) {
            // Check if we've stopped at an entity
            if (tokens[t].token) {
                // Examine the consecutive entities, if any.
                for (var _i = t + 1; _i < tokens.length; _i++) {
                    // Combine consecutive entities of the same type into one entity. Then, mark the duplicates as 'false'.
                    if (typeof tokens[_i] !== 'string' && tokens[_i].type === tokens[t].type) {
                        tokens[t].token += ' ' + tokens[_i].token;
                        tokens[_i] = false;
                    }
                    // Stop the loop when we've run out of consecutive entities
                    if (typeof tokens[_i] === 'string') {
                        break;
                    }
                }
            }
        }
        // Filter out the consecutive entities that were marked as duplicates
        tokens = tokens.filter(function (val) {
            return !!val;
        });
        // Add a space to the end of each string/non-entity
        var tokensWithSpaces = tokens.map(function (t) {
            if (typeof t === 'string') {
                return t + ' ';
            }
            return t;
        });
        // Loop through our 'tokens' array. Push strings directly to the 'jsx' array. Convert entity objects to jsx markup, then push to the 'jsx' array.
        tokensWithSpaces.forEach(function (t) {
            if (typeof t === 'string') {
                jsx.push(t);
            } else {
                jsx.push(_react2.default.createElement(
                    'mark',
                    {
                        style: {
                            padding: '0.25em 0.35em',
                            margin: '0px 0.25em',
                            lineHeight: '1',
                            display: 'inline-block',
                            borderRadius: '0.25em',
                            border: '1px solid',
                            background: 'rgba(\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.r + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.g + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.b + ',\n                                0.2\n                            )',
                            borderColor: 'rgb(\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.r + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.g + ',\n                                ' + ents.find(function (ent) {
                                return ent.type === t.type;
                            }).color.b + '\n                            )'
                        }
                    },
                    t.token,
                    _react2.default.createElement(
                        'span',
                        {
                            style: {
                                boxSizing: 'border-box',
                                fontSize: '0.6em',
                                lineHeight: '1',
                                padding: '0.35em',
                                borderRadius: '0.35em',
                                textTransform: 'uppercase',
                                display: 'inline-block',
                                verticalAlign: 'middle',
                                margin: '0px 0px 0.1rem 0.5rem',
                                background: 'rgb(\n                                    ' + ents.find(function (ent) {
                                    return ent.type === t.type;
                                }).color.r + ',\n                                    ' + ents.find(function (ent) {
                                    return ent.type === t.type;
                                }).color.g + ',\n                                    ' + ents.find(function (ent) {
                                    return ent.type === t.type;
                                }).color.b + '\n                                )'
                            }
                        },
                        t.type
                    )
                ));
            }
        });
    }

    // Return the markup
    return _react2.default.createElement(
        'div',
        null,
        jsx.map(function (j, i) {
            return _react2.default.createElement(
                'span',
                { key: i },
                j
            );
        })
    );
};

exports.default = Taggy;